unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, //TeeGDIPlus,
  ExtCtrls, TeeProcs, TeEngine, Chart, Series, TeeTools;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure Chart1Zoom(Sender: TObject);
    procedure Chart1UndoZoom(Sender: TObject);
    procedure Chart1Scroll(Sender: TObject);
    procedure Chart1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);

    procedure FormResize(Sender: TObject);


  private
    { Private declarations }
    // makes marks draggable
    procedure DragMarksOnDragged(Sender:TDragMarksTool; Index:Integer;
                                    Button:TMouseButton; Shift: TShiftState;
                                    X,Y: Integer);

    procedure ScrollBar1Change(Sender : TObject);

    procedure Chart1Resize(Sender: TObject);

    procedure PlaceMarks2;

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses StdCtrls;

{$R *.dfm}

var Chart1: TChart;
    ScrollBar1 : TScrollBar;
    XOffset, YOffset: Array of Integer;
    MouseDownPos: TPoint;


procedure TForm1.FormCreate(Sender: TObject);
var i: Integer;
min, max : Integer;

begin
  Chart1:=TChart.Create(Self);
  Chart1.Parent:=Self;
  Chart1.Align:=alClient;

  Chart1.Legend.Visible:=false;
  Chart1.View3D:=false;

  min := 0;
  max := 50;


  with Chart1.AddSeries(TPointSeries) do
  begin
    FillSampleValues(50);
    ColorEachPoint:=true;
    Marks.Arrow.Color:=clRed;
    Marks.Visible := True;
    Marks.ArrowLength := 20;
  end;

  SetLength(XOffset, Chart1[0].Count);
  SetLength(YOffset, Chart1[0].Count);

  for i:=0 to High(XOffset) do
  begin
    XOffset[i]:=0;
    YOffset[i]:=0;
  end;

  with Chart1.Tools.Add(TDragMarksTool) as TDragMarksTool do
  begin
    Series:=Chart1[0];
    OnDraggedMark:=DragMarksOnDragged;
  end;

  Chart1.AllowPanning := pmBoth;
  Chart1.Zoom.Allow := false;
  Chart1.OnScroll:=Chart1Scroll;

  chart1.Axes.Bottom.Increment := 1;
  chart1.Axes.Bottom.Automatic := False;
  chart1.Axes.Bottom.Minimum := min;
  chart1.Axes.Bottom.Maximum := max;

  Scrollbar1 := TScrollBar.Create(Chart1);
  Scrollbar1.Parent := Chart1;
  ScrollBar1.Align := alBottom;
  ScrollBar1.OnChange := ScrollBar1Change;

  scrollBar1.SetParams(0, min, max);

  scrollBar1.PageSize := Trunc(max/3);
  scrollBar1.SmallChange := 2;
  scrollBar1.LargeChange := 4;
  scrollBar1.Position := 0;
  scrollBar1.Visible := true;
  scrollbar1.Enabled := true;

  Chart1.OnResize := Chart1Resize;

  Self.ScrollBar1Change(nil);
end;



procedure TForm1.PlaceMarks2;
var s, i, //tmpWidth, tmpHeight,
tmpX, tmpY: Integer;
  series : TPointSeries;
  markPos : TSeriesMarkPosition;

begin
  Chart1.Draw();

  for S := 0 to Chart1.SeriesCount - 1 do
  begin
    if Chart1[S] is TPointSeries then
    begin
      Series := Chart1[S] as TPointSeries;
        for i := 0 to Series.Count - 1 do
        begin
          if (Series.Marks.Positions[i] <> nil) and (Series.Marks.Positions[i].Custom) then
          begin

            markPos := Series.Marks.Positions[i]; // saved custom location in screen coords
            tmpX := Series.CalcXPos(i);  // where the point x, y is now, in screen coords
            tmpY := Series.CalcYPos(i);
            markPos.ArrowFrom := Point(tmpX, tmpY);

            markPos.LeftTop.X := tmpX + xOffset[i];
            markPos.LeftTop.Y := tmpY + yOffset[i];
            markPos.ArrowTo := Point(markPos.LeftTop.X, markPos.LeftTop.Y);
          end;
        end;
    end;
  end;
end;




procedure TForm1.ScrollBar1Change(Sender: TObject);
var
  graphMin : Double;
  graphMax : Double;

begin
  graphMin := scrollBar1.Position;
  graphMax := graphMin + scrollBar1.PageSize;

  if graphMax > scrollBar1.Max then
  begin
    graphMax := scrollBar1.Max;
  end;

  if graphMin < scrollBar1.Min then
  begin
    graphMin := scrollBar1.Min;
  end;

  chart1.Axes.Bottom.SetMinMax(graphMin, graphMax);

  PlaceMarks2;
end;

procedure TForm1.DragMarksOnDragged(Sender:TDragMarksTool; Index:Integer;
                                    Button:TMouseButton; Shift: TShiftState;
                                    X,Y: Integer);
begin
 //Collect the updated positions.

  if Assigned(Sender.Series) then
  begin
    if (Index>-1) and (Index<Length(XOffset)) then
    begin
       // this calculation appears to be wrong!

       //nope
      //XOffset[Index]:=XOffset[Index]+X-MouseDownPos.X;
      //YOffset[Index]:=YOffset[Index]+Y-MouseDownPos.Y;

      //nope
      //XOffset[Index]:=XOffset[Index]+X;
      //YOffset[Index]:=YOffset[Index]+Y;

      //nope
      //XOffset[Index]:=X;
      //YOffset[Index]:=Y;

      //nope
      //XOffset[Index]:=Sender.Series.Marks.Positions[Index].LeftTop.X-X;
      //YOffset[Index]:=Sender.Series.Marks.Positions[Index].LeftTop.Y-Y;



    end;
  end;
end;

procedure TForm1.Chart1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
// In the act of zooming or scrolling using the mouse drag way, this is needed to calculate
// XScroll YScroll so it moves with each mouse point.
// It also gets called when a mark is dragged, but I don't think it's helpful?
  MouseDownPos.X:=X;
  MouseDownPos.Y:=Y;
end;




procedure TForm1.Chart1Resize(Sender: TObject);
begin
// chart handles updating mark positions correctly, but
// handles custom marks incorrectly.
end;


procedure TForm1.FormResize(Sender: TObject);
begin
// occurs after the form is resized. No need to do anything.
end;

procedure TForm1.Chart1Scroll(Sender: TObject);
begin
// never called when we use the scrollbar rather than dragging the mouse

//This is for the weird scroll way. You hold the mouse down and drag the
//direction you want to scroll.
//  PlaceMarks2;
end;

procedure TForm1.Chart1Zoom(Sender: TObject);
begin
// never called when we use the scrollbar rather than dragging the mouse
//PlaceMarks2;
end;

procedure TForm1.Chart1UndoZoom(Sender: TObject);
begin
// never called when we use the scrollbar rather than dragging the mouse
//PlaceMarks2;
end;



end.
